#include	<File.h>
#include	<ctype.h>
#include	<stdio.h>
#include	<InterfaceKit.h>
#include	"AKIROMRS.h"

#define	BLANKCHECK_MSG		'msg0'
#define	TRANSMIT_MSG		'msg1'
#define	WRITERAM_MSG		'msg2'
#define	WANDT_MSG		'msg3'
#define	READRAM_MSG		'msg4'
#define	READRCV_MSG		'msg5'
#define	CMPFILE_MSG		'msg6'
#define	CMPRAM_MSG		'msg7'

unsigned int	HexToInt(const char *hex)
{
int	i;
int	a = 1;
int	result = 0;
char	h;

	i = strlen( hex ) -1;
	for(;i>=0;i--){
		if (isxdigit(hex[i]) != 0) {
			h = toupper(hex[i]);
			switch (h) {
				case '0': break;
				case '1': result += 1 * a;
						break;
				case '2': result += 2 * a;
						break;
				case '3': result += 3 * a;
						break;
				case '4': result += 4 * a;
						break;
				case '5': result += 5 * a;
						break;
				case '6': result += 6 * a;
						break;
				case '7': result += 7 * a;
						break;
				case '8': result += 8 * a;
						break;
				case '9': result += 9 * a;
						break;
				case 'A': result += 0xa * a;
						break;
				case 'B': result += 0xb * a;
						break;
				case 'C': result += 0xc * a;
						break;
				case 'D': result += 0xd * a;
						break;
				case 'E': result += 0xe * a;
						break;
				case 'F': result += 0xf * a;
						break;
				default:  break;
			}
		} else {
			result = -1;
			break;
		};
		a = a * 0x10;
	};
	return result;
}

TAKIROMRS::TAKIROMRS(BLooper *Target,BMessage *Message):BLooper("AKIROMLooper")
{
	if ( Target == NULL ) {
		MessageTarget = be_app;
	} else {
		MessageTarget = Target;
	};
	InfoMessage = Message;
	InfoMessage->AddString("info","dummy");
	InfoMessage->AddInt32("proc",0);
	InfoMessage->AddBool("endf",false);

	RomType = 2;
	StartAdr = 0;
	EndAdr = 0x7FFF;
	Size = 0x8000;
	Batch = false;

	serial.SetFlowControl(B_HARDWARE_CONTROL);
	serial.SetDataRate(B_9600_BPS);
	serial.SetDataBits(B_DATA_BITS_8);
	serial.SetStopBits(B_STOP_BITS_1);
	serial.SetParityMode(B_NO_PARITY);
	serial.SetTimeout(5000000);
	serial.SetBlocking(true);
	
	BlankCheckMsg = new BMessage(BLANKCHECK_MSG);
	TransmitMsg = new BMessage(TRANSMIT_MSG);
	WriteRamMsg = new BMessage(WRITERAM_MSG);
	WandTMsg = new BMessage(WANDT_MSG);
	ReadRAMMsg = new BMessage(READRAM_MSG);
	ReadRcvMsg = new BMessage(READRCV_MSG);
	CmpFileMsg = new BMessage(CMPFILE_MSG);
	CmpRAMMsg = new BMessage(CMPRAM_MSG);

	for (int i=0;i<32768;i++) {
		RomBuff[i] = 0xFF;
	};
}

TAKIROMRS::~TAKIROMRS()
{
	delete BlankCheckMsg;
	delete TransmitMsg;
	delete WriteRamMsg;
	delete WandTMsg;
	delete ReadRAMMsg;
	delete ReadRcvMsg;
	delete CmpFileMsg;
	delete CmpRAMMsg;
}

void	TAKIROMRS::BlankCheck()
{
	PostMessage(BlankCheckMsg);
}

void	TAKIROMRS::TransmitToAKIRAM()
{
	PostMessage(TransmitMsg);
}

void	TAKIROMRS::WritefromAKIRAM()
{
	PostMessage(WriteRamMsg);
}

void	TAKIROMRS::WriteAndTransmit()
{
	PostMessage(WandTMsg);
}

void	TAKIROMRS::ReadToAKIRAM()
{
	PostMessage(ReadRAMMsg);
}

void	TAKIROMRS::ReadAndReceive()
{
	PostMessage(ReadRcvMsg);
}

void	TAKIROMRS::CompareFile()
{
	PostMessage(CmpFileMsg);
}

void	TAKIROMRS::CompareAKIRAM()
{
	PostMessage(CmpRAMMsg);
}

void	TAKIROMRS::MessageReceived(BMessage *msg)
{
	switch( msg->what ) {
		case	BLANKCHECK_MSG:
			_BlankCheck();
			break;
		case	TRANSMIT_MSG:
			_TransmitToAKIRAM();
			break;
		case	WRITERAM_MSG:
			_WritefromAKIRAM();
			break;
		case	WANDT_MSG:
               _WriteAndTransmit();
			break;
		case	READRAM_MSG:
			_ReadToAKIRAM();
			break;
		case	READRCV_MSG:
			_ReadAndReceive();
			break;
		case	CMPFILE_MSG:
			_CompareFile();
			break;
		case	CMPRAM_MSG:
			_CompareAKIRAM();
			break;
	};
}

int	TAKIROMRS::SetStartEndAdr(int SAdr,int EAdr)
{
int	result;

	result = CheckAdr(SAdr,EAdr);
	if ( result > -1 ) {
		StartAdr = SAdr;
		EndAdr = EAdr;
		Size = result;
	};
	return result;
}

int	TAKIROMRS::GetStartAdr()
{
	return StartAdr;
}

int	TAKIROMRS::GetEndAdr()
{
	return EndAdr;
}

int	TAKIROMRS::GetSize()
{
	return Size;
}

int	TAKIROMRS::CheckAdr(int sa,int ea)
{
	Size = ea - sa +1;
	if ( Size <= 0) {
		Size = -1;
		return Size;
	};
	if ( RomType == i27256 ) {
		if ( ( sa > 0x7FFF ) | ( ea > 0x7FFF ) ) {
			Size = -1;
			return Size;
		};
	};
	if ( RomType == i27128 ) {
		if ( ( sa > 0x3FFF ) | ( ea > 0x3FFF ) ) {
			Size = -1;
			return Size;
		};
	};
	if ( RomType == i2764 ) {
		if ( ( sa > 0x1FFF ) | ( ea > 0x1FFF ) ) {
			Size = -1;
			return Size;
		};
	};
	if ( RomType == i2864 ) {
		if ( ( sa > 0x1FFF ) | ( ea > 0x1FFF ) ) {
			Size = -1;
			return Size;
		};
	};
	return Size;
}

void	TAKIROMRS::AKIProcessStart( char *msg )
{
	InfoMessage->ReplaceString("info",msg);
	InfoMessage->ReplaceBool("endf",false);
	MessageTarget->PostMessage(InfoMessage);
}

void	TAKIROMRS::AKIProcessEnd( char *msg,int result )
{
	InfoMessage->ReplaceString("info",msg);
	InfoMessage->ReplaceInt32("proc",result);
	InfoMessage->ReplaceBool("endf",!Batch);
	MessageTarget->PostMessage(InfoMessage);
}

void	TAKIROMRS::AKIProcessAbort( int errcode )
{
	switch ( errcode ) {
		case AKI_OPEN_ERR:
			AKIProcessEnd( "Open Error!",errcode );
			break;
		case AKI_RS_NOTRDY:
			AKIProcessEnd( "AKI-ROMWriter Not Ready!",errcode );
			break;
		case AKI_NOT_RDY:
			AKIProcessEnd( "Not AKI-ROMWriter!",errcode );
			break;
		case AKI_INV_ROM:
			AKIProcessEnd( "ROM Type Invalid!",errcode );
			break;
		case AKI_INV_ADR:
			AKIProcessEnd( "Start/End Address Invalid!",errcode );
			break;
		default:
			AKIProcessEnd( "????",errcode );
			break;
	};
}

int	TAKIROMRS::AKICheckOpen()
{
char	buff[22];
char	AKIRES[23] = "Yes My Name is AKI-80\0";

	if ( serial.Open( devName ) <=0 ) {
		return AKI_OPEN_ERR;
	};

	if ( Send("Are You AKI-80 ?\r\n",18) != 18 ) {
		serial.Close();
		return AKI_RS_NOTRDY;
	};
	Recv(buff,21);
	printf("AKIROM result string = %s\n",buff);
	if ( strcmp( AKIRES,buff ) !=0 ) {
		serial.Close();
		return AKI_NOT_RDY;
	};

	return AKI_OK;
}

int	TAKIROMRS::_BlankCheck()
{
char		buff[3];
char		amsg[256];
int		result;

	AKIProcessStart("Blank Checking...");

	result = AKICheckOpen();
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		return result;
	};
	result = romCmd( CBlankCheck );
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};
	result = txSize();
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};
	if ( Recv(buff,2) != 2 ) {
		AKIProcessAbort( AKI_RS_NOTRDY );
		serial.Close();
		return result;
	} else {
		result = buff[0] | buff[1];
		if ( result != 0 ) {
			sprintf(amsg,"Blank Check %dbytes Error", ( (0xFF & buff[0]) << 8) | (0xFF & buff[1]) );
		} else {
			sprintf(amsg,"Blank Check OK!");
		};
	};
	AKIProcessEnd(amsg,AKI_OK);
	serial.Close();
	if ( result != 0 ) {
		return AKI_NOT_BLANKROM;
	} else {
		return AKI_OK;
	};
}

int	TAKIROMRS::_TransmitToAKIRAM()
{
int	result,i;
char	amsg[256];

	AKIProcessStart("Transmitting...");

	if ( Size == -1 ) {
		AKIProcessAbort(AKI_INV_ADR);
		return AKI_INV_ADR;
	};
	result = AKICheckOpen();
	if ( result != AKI_OK ) {
		AKIProcessAbort(result);
		return result;
	};
	result = romCmd( CTXtoAKIRAM );
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};
	result = txSize();
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};

	for ( i = 0 ; i<Size ; i++ ) {
		if ( Send( RomBuff + i,1 ) != 1 ) {
			AKIProcessAbort( AKI_RS_NOTRDY );
			serial.Close();
			return AKI_RS_NOTRDY;
		};
		sprintf(amsg,"Transmitting %dbytes",i);
		InfoMessage->ReplaceString("info",amsg);
		InfoMessage->ReplaceInt32("proc",i+1);
		MessageTarget->PostMessage(InfoMessage);
	};
	sprintf( amsg,"Transmit %dbytes Completed!",i );
	AKIProcessEnd(amsg,AKI_OK);
	serial.Close();
	return AKI_OK;
}

int	TAKIROMRS::_WritefromAKIRAM()
{
int	i,result;
char	c[2];
char	amsg[256];

	AKIProcessStart("Writing...");

	if ( Size == -1 ) {
		AKIProcessAbort(AKI_INV_ADR);
		return AKI_INV_ADR;
	};
	result = AKICheckOpen();
	if ( result != AKI_OK ) {
		AKIProcessAbort(result);
		return result;
	};
	result = romCmd( CWRfromAKIRAM );
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};
	result = txSize();
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};
	for(i=16;;i=i+16){
		if ( Recv(c,1) == 1 ) {
			if ( ( c[0] == 0) | ( c[0] == -1 ) ) { break; };
			if ( c[0] == 1 ) {
				sprintf(amsg,"Writing %dbytes",i);
				InfoMessage->ReplaceString("info",amsg);
				InfoMessage->ReplaceInt32("proc",i);
				MessageTarget->PostMessage(InfoMessage);
			};
		} else {
			AKIProcessAbort( AKI_RS_NOTRDY );
			serial.Close();
			return AKI_RS_NOTRDY;
		};
	};
	sprintf(amsg,"Writing %dbytes Completed",i);
	AKIProcessEnd(amsg,AKI_OK);
	serial.Close();
	return AKI_OK;
}

int	TAKIROMRS::_WriteAndTransmit()
{
int	result;

	Batch = true;
	result = _BlankCheck();
	if ( result != AKI_OK ) {
		Batch = false;
		AKIProcessAbort( result );
		return result;
	};
	result = _TransmitToAKIRAM();
	if ( result != AKI_OK ) {
		Batch = false;
		AKIProcessAbort( result );
		return result;
	};
	result = _WritefromAKIRAM();
	if ( result != AKI_OK ) {
		Batch = false;
		AKIProcessAbort( result );
		return result;
	};
	Batch = false;
	AKIProcessEnd("Complete!",AKI_OK);
	return AKI_OK;
}

int	TAKIROMRS::_ReadToAKIRAM()
{
char	buff[3];
int	result;
char	amsg[256];

	AKIProcessStart("Read Start");

	result = AKICheckOpen();
	if ( result != AKI_OK ) {
		AKIProcessAbort(result);
		return result;
	};
	result = romCmd( CRDtoAKIRAM );
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};
	result = txSize();
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};

	if ( Recv(buff,2) != 2 ) {
		AKIProcessAbort( AKI_RS_NOTRDY );
		serial.Close();
		return result;
	} else {
		if ( buff[0] | buff[1] ) {
			sprintf(amsg,"AKIRAM LoadError at 0x%2.2x%2.2x\n", 0xFF & buff[0], 0xFF & buff[1]);
		} else {
			sprintf(amsg,"AKIRAM Load OK");
		};
	};

	AKIProcessEnd(amsg,AKI_OK);
	serial.Close();
	return AKI_OK;
}

int	TAKIROMRS::_ReadAndReceive()
{
char	buff[3];
int	result,i;
char	amsg[256];

	AKIProcessStart("Receive Start");

	result = AKICheckOpen();
	if ( result != AKI_OK ) {
		AKIProcessAbort(result);
		return result;
	};
	result = romCmd( CRDandRX );
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};
	result = txSize();
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};

	for(i=StartAdr;i<EndAdr;i++){
		if ( Recv(buff,1) == 1 ) {
			RomBuff[i] = buff[0];
			sprintf(amsg,"Receiving %dbytes",i-StartAdr);
			InfoMessage->ReplaceString("info",amsg);
			InfoMessage->ReplaceInt32("proc",i-StartAdr);
			MessageTarget->PostMessage(InfoMessage);
		} else {
			AKIProcessAbort( AKI_RS_NOTRDY );
			serial.Close();
			return AKI_RS_NOTRDY;
		};
	};
	sprintf(amsg,"%dbytes Received",Size);
	AKIProcessEnd(amsg,AKI_OK);
	serial.Close();
	return AKI_OK;
}

int	TAKIROMRS::_CompareAKIRAM()
{
char	buff[3];
int	result;
char	amsg[256];

	AKIProcessStart("Compare Start");

	result = AKICheckOpen();
	if ( result != AKI_OK ) {
		AKIProcessAbort(result);
		return result;
	};
	result = romCmd( CCMP_AKIRAM );
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};
	result = txSize();
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};

	if ( Recv(buff,2) != 2 ) {
		AKIProcessAbort( AKI_NOT_RDY );
		serial.Close();
		return result;
	} else {
		if ( buff[0] | buff[1] ) {
			sprintf(amsg,"AKIRAM CompareError %dbytes", ( (0xFF & buff[0]) << 8) | (0xFF & buff[1]) );
		} else {
			sprintf(amsg,"AKIRAM Compare OK");
		};
	};

	AKIProcessEnd(amsg,AKI_OK);
	serial.Close();
	return AKI_OK;
}

int	TAKIROMRS::_CompareFile()
{
char	buff[3];
int	result,i;
char	amsg[256];
int	count = 0;

	AKIProcessStart("Compare Start");

	result = AKICheckOpen();
	if ( result != AKI_OK ) {
		AKIProcessAbort(result);
		return result;
	};
	result = romCmd( CRDandRX );
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};
	result = txSize();
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};

	for(i=StartAdr;i<EndAdr;i++){
		if ( Recv(buff,1) == 1 ) {
			sprintf(amsg,"Comparing %dbytes",i-StartAdr);
			InfoMessage->ReplaceString("info",amsg);
			InfoMessage->ReplaceInt32("proc",i-StartAdr);
			MessageTarget->PostMessage(InfoMessage);
			if ( RomBuff[i] != buff[0] ) {
				count++;
			};
		} else {
			AKIProcessAbort( AKI_RS_NOTRDY );
			serial.Close();
			return AKI_RS_NOTRDY;
		};
	};
	sprintf(amsg,"Compare Error %dbytes",count);
	AKIProcessEnd(amsg,AKI_OK);
	serial.Close();
	return AKI_OK;
}

int	TAKIROMRS::CheckSum()
{
char	buff[3];
int	result;
char	amsg[256];

	AKIProcessStart("Check Start");

	result = AKICheckOpen();
	if ( result != AKI_OK ) {
		AKIProcessAbort(result);
		return result;
	};
	result = romCmd( CCSUM );
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};
	result = txSize();
	if ( result != AKI_OK ) {
		AKIProcessAbort( result );
		serial.Close();
		return result;
	};

	if ( Recv(buff,2) != 2 ) {
		AKIProcessAbort( AKI_NOT_RDY );
		serial.Close();
		return result;
	} else {
		result = ( buff[0] << 8 ) | (0xff & buff[1]);
		sprintf(amsg,"Check Sum = %d",result);
	};

	AKIProcessEnd(amsg,AKI_OK);
	return result;
}

int	TAKIROMRS::romCmd( char cmd )
{
char	t[1];

	switch( RomType ) {
		case 0:	t[0] = 0x10 | cmd; break;
		case 1:	t[0] = 0x20 | cmd; break;
		case 2:	t[0] = 0x30 | cmd; break;
		case 3:	t[0] = ( ((cmd==1)||(cmd==2))? 0x40 : 0x10 ) | cmd; break;
		default:	return AKI_INV_ROM;
	}
	if ( Send( t,1 ) != 1 ) return AKI_RS_NOTRDY;
	return AKI_OK;
}

int	TAKIROMRS::txSize()
{
char	sbuf[4];

	if ( Size == -1 ) return AKI_INV_ADR;
	sbuf[0] = ( StartAdr >> 8 ) & 0xff;
	sbuf[1] = StartAdr & 0xff;
	sbuf[2] = ( Size >> 8 ) & 0xff;
	sbuf[3] = Size & 0xff;
	if ( Send( sbuf,4 ) != 4 ) return AKI_RS_NOTRDY;
	return AKI_OK;
}

int	TAKIROMRS::Recv( char *s,int i )
{
int	r = 0;
char	x[1];

	while (i>r) {
		if (serial.Read(x,1) == 1) {
			s[r]=x[0];
			r++;
		} else {
			break;
		};
	};
	s[r] = 0;
	return r;
}

int	TAKIROMRS::Send( char *s,int i )
{
int	r = 0;

	while ( (i>r) & (serial.IsCTS()==true) ) {
		serial.Write( s+r,1 );
		r++;
	};
	return r;
}

int	TAKIROMRS::LoadBinFile(entry_ref *ref)
{
int		result;
BFile		file;

	if ( file.SetTo( ref , B_READ_ONLY ) != B_OK ) { return -1; }; 
	result = file.Read( RomBuff,32768 );
	file.Unset();
	return result;
}

int	TAKIROMRS::LoadHexFile(entry_ref *ref)
{
int		result;
bool		eof;
char		buf[5];
int		i;
int		Length;
int		Adr;
int		Type;
int		CSum;
int		Sum;
int		Data[256];
BFile		file;
BAlert	*alert;

	if ( file.SetTo( ref , B_READ_ONLY ) != B_OK ) { return -1; }; 
	result = 0;
	buf[0] = 0;
	eof = false;

	while ( eof == false ) {
		CSum = 0;

/*   ":" check      */
		while ( buf[0] != 0x3a ) {
			if ( file.Read( buf,1 ) != 1 ) {
				result = -1;
				break;
			};
		};

/*   Length check   */
		if ( file.Read( buf,2 ) != 2 ) {
			result = -2;
			break;
		};
		buf[2] = 0;
		Length = HexToInt( buf );
		if ( Length == -1 ) {
			result = -3;
			break;
		};
		CSum += Length;
		printf("\nLength = %x\n",Length);

/*   Address        */
		if ( file.Read( buf,4 ) != 4 ) {
			result = -4;
			break;
		};
		buf[4] = 0;
		Adr = HexToInt( buf );
		if ( Adr == -1 ) {
			result = -5;
			break;
		};
		CSum += ( ( Adr & 0xFF ) + ( Adr >> 8 ) );
		printf("Adr = %x\n",Adr);

/*   Record Type    */
		if ( file.Read( buf,2 ) != 2 ) {
			result = -6;
			break;
		};
		buf[2] = 0;
		Type = HexToInt( buf );
		if ( Type == -1 ) {
			result = -7;
			break;
		};
		CSum += Type;
		printf("Type = %x\n",Type);

/*   Data            */

		printf("Length = %x\n",Length);

		if ( Type == 0 ) {
			for (i=0;Length>i;i++) {
				if ( file.Read( buf,2 ) != 2 ) {
					result = -8;
					printf("HexFile Error (-8)  %s\n",buf);
					break;
				};
				buf[2] = 0;
				Data[i] = HexToInt( buf );
				if ( Data[i] == -1 ) {
					result = -9;
					printf("HexFile Error (-9)  %s\n",buf);
					break;
				};
				printf("Data[%x] = %x\n",i,0xff & Data[i]);
				CSum += Data[i];
			};
		} else {
			result = 0;
			break;
		};
		printf( "+CSum = %x\n" , CSum );
		CSum = 0xFF & (0x100 - ( CSum & 0xFF ) );

/*   Check Sum       */
		if ( file.Read( buf,2 ) != 2 ) {
			result = -10;
			break;
		};
		buf[2] = 0;
		Sum = HexToInt( buf );

		if ( Sum == -1 ) {
			result = -11;
			break;
		};
		printf( "CSum = %x  Sum = %x  File = %s\n",CSum,Sum,buf );
		if ( Sum != CSum ) {
			alert = new BAlert("","Check Sum Error","OK");
			alert->Go();
			result = -12;
			break;
		};

		for (i=0;Length>i;i++) {
			RomBuff[ Adr + i ] = 0xFF & Data[i];
		};
	};
	file.Unset();
	printf("* HexFile Loaded Status = %d\n",result);
	return result;
}

int	TAKIROMRS::SaveBinFile(const char *path)
{
int		result;
BFile		file;

	if ( file.SetTo( path , B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE ) != B_OK ) { return -1; };
	result = file.Write( RomBuff,32768 );
	file.Unset();
	return result;
}

int	TAKIROMRS::SaveHexFile(const char *path)
{
int		result;
char		amsg[256];
BFile		file;
unsigned	int	i,Sum;

	result = 0;
	if ( file.SetTo( path , B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE ) != B_OK ) { return -1; };
	for (i=0;i<32768;i+=32) {
		Sum = 0xFF & ( 0x100 - ( ( 0x20 + ( i & 0xFF ) + ( i >> 8 ) +
			   RomBuff[i+ 0] + RomBuff[i+ 1] + RomBuff[i+ 2] + RomBuff[i+ 3] +
		        RomBuff[i+ 4] + RomBuff[i+ 5] + RomBuff[i+ 6] + RomBuff[i+ 7] +
		        RomBuff[i+ 8] + RomBuff[i+ 9] + RomBuff[i+10] + RomBuff[i+11] +
		        RomBuff[i+12] + RomBuff[i+13] + RomBuff[i+14] + RomBuff[i+15] +
			   RomBuff[i+16] + RomBuff[i+17] + RomBuff[i+18] + RomBuff[i+19] +
		        RomBuff[i+20] + RomBuff[i+21] + RomBuff[i+22] + RomBuff[i+23] +
		        RomBuff[i+24] + RomBuff[i+25] + RomBuff[i+26] + RomBuff[i+27] +
		        RomBuff[i+28] + RomBuff[i+29] + RomBuff[i+30] + RomBuff[i+31] ) & 0xFF ) );
		sprintf(amsg,":20%4.4X00%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X\r\n",
		         i,
		         0xFF & RomBuff[i+ 0],0xFF & RomBuff[i+ 1],0xFF & RomBuff[i+ 2],0xFF & RomBuff[i+ 3],
		         0xFF & RomBuff[i+ 4],0xFF & RomBuff[i+ 5],0xFF & RomBuff[i+ 6],0xFF & RomBuff[i+ 7],
		         0xFF & RomBuff[i+ 8],0xFF & RomBuff[i+ 9],0xFF & RomBuff[i+10],0xFF & RomBuff[i+11],
		         0xFF & RomBuff[i+12],0xFF & RomBuff[i+13],0xFF & RomBuff[i+14],0xFF & RomBuff[i+15],
		         0xFF & RomBuff[i+16],0xFF & RomBuff[i+17],0xFF & RomBuff[i+18],0xFF & RomBuff[i+19],
		         0xFF & RomBuff[i+20],0xFF & RomBuff[i+21],0xFF & RomBuff[i+22],0xFF & RomBuff[i+23],
		         0xFF & RomBuff[i+24],0xFF & RomBuff[i+25],0xFF & RomBuff[i+26],0xFF & RomBuff[i+27],
		         0xFF & RomBuff[i+28],0xFF & RomBuff[i+29],0xFF & RomBuff[i+30],0xFF & RomBuff[i+31],
		         Sum);
		if ( file.Write(amsg,strlen(amsg)) != 77 ) {
			result = -1;
			break;
		};
	};

	if ( result == 0 ) {
		sprintf(amsg,":00000001FF\r\n");
		if ( file.Write(amsg,strlen(amsg)) != 13 ) {
			result = -1;
		};
	};

	if ( result != 0 ) {
		BAlert *alert = new BAlert("","Write error","OK");
		alert->Go();
	};

	file.Unset();
	return result;
}
